# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: messages.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='messages.proto',
  package='robotc',
  syntax='proto3',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x0emessages.proto\x12\x06robotc\"\x17\n\x08Response\x12\x0b\n\x03ret\x18\x01 \x01(\x05\"-\n\nCoordinate\x12\t\n\x01x\x18\x01 \x01(\x01\x12\t\n\x01y\x18\x02 \x01(\x01\x12\t\n\x01z\x18\x03 \x01(\x01\"+\n\x08Rotation\x12\t\n\x01r\x18\x01 \x01(\x01\x12\t\n\x01p\x18\x02 \x01(\x01\x12\t\n\x01y\x18\x03 \x01(\x01\"N\n\x02PR\x12$\n\x08position\x18\x01 \x01(\x0b\x32\x12.robotc.Coordinate\x12\"\n\x08rotation\x18\x02 \x01(\x0b\x32\x10.robotc.Rotation\"\x17\n\x05JPose\x12\x0e\n\x06joints\x18\x01 \x03(\x01\"8\n\x07Payload\x12\x0c\n\x04mass\x18\x01 \x01(\x01\x12\x1f\n\x03\x63og\x18\x02 \x01(\x0b\x32\x12.robotc.Coordinate\"\x1b\n\x0bPayloadMass\x12\x0c\n\x04mass\x18\x01 \x01(\x01\"-\n\nPayloadCog\x12\x1f\n\x03\x63og\x18\x01 \x01(\x0b\x32\x12.robotc.Coordinate\"\x14\n\x05IOPin\x12\x0b\n\x03pin\x18\x01 \x01(\x05\"!\n\x03\x44IO\x12\x0b\n\x03pin\x18\x01 \x01(\x05\x12\r\n\x05value\x18\x02 \x01(\x05\"/\n\x03\x41IO\x12\x0b\n\x03pin\x18\x01 \x01(\x05\x12\x0c\n\x04mode\x18\x02 \x01(\x05\x12\r\n\x05value\x18\x03 \x01(\x01\"2\n\x15ModbusExternalIOState\x12\n\n\x02id\x18\x01 \x01(\x05\x12\r\n\x05state\x18\x02 \x01(\x05\"\xfe\x01\n\x02IO\x12\x1f\n\nrobotDIOIn\x18\x01 \x03(\x0b\x32\x0b.robotc.DIO\x12 \n\x0brobotDIOOut\x18\x02 \x03(\x0b\x32\x0b.robotc.DIO\x12\x1f\n\nrobotAIOIn\x18\x03 \x03(\x0b\x32\x0b.robotc.AIO\x12 \n\x0brobotAIOOut\x18\x04 \x03(\x0b\x32\x0b.robotc.AIO\x12\x1d\n\x08tcpDIOIn\x18\x05 \x03(\x0b\x32\x0b.robotc.DIO\x12\x1e\n\ttcpDIOOut\x18\x06 \x03(\x0b\x32\x0b.robotc.DIO\x12\x33\n\x0c\x65xtIODevices\x18\x07 \x03(\x0b\x32\x1d.robotc.ModbusExternalIOState\"=\n\x08Hardware\x12\r\n\x05model\x18\x01 \x01(\t\x12\n\n\x02sn\x18\x02 \x01(\t\x12\x16\n\x0e\x64river_version\x18\x03 \x01(\t\"\xa1\x02\n\tRobotInfo\x12\x1f\n\x05robot\x18\x01 \x01(\x0b\x32\x10.robotc.Hardware\x12 \n\x06\x66lange\x18\x02 \x01(\x0b\x32\x10.robotc.Hardware\x12\x1d\n\x03led\x18\x03 \x01(\x0b\x32\x10.robotc.Hardware\x12 \n\x06joints\x18\x04 \x03(\x0b\x32\x10.robotc.Hardware\x12\x11\n\trcVersion\x18\x05 \x01(\t\x12\x0c\n\x04name\x18\x06 \x01(\t\x12\"\n\x08\x63omboard\x18\x07 \x01(\x0b\x32\x10.robotc.Hardware\x12\x0b\n\x03mac\x18\x08 \x01(\t\x12\x1d\n\x03\x61rm\x18\t \x01(\x0b\x32\x10.robotc.Hardware\x12\x0e\n\x06\x61ppTag\x18\n \x01(\t\x12\x0f\n\x07typeTag\x18\x0b \x01(\t\"A\n\nDeviceInfo\x12!\n\x07\x64\x65vInfo\x18\x01 \x01(\x0b\x32\x10.robotc.Hardware\x12\x10\n\x08\x64\x65v_type\x18\x02 \x01(\x05*[\n\x08LuaState\x12\x08\n\x04IDLE\x10\x00\x12\x0b\n\x07RUNNING\x10\x01\x12\n\n\x06PAUSED\x10\x02\x12\x0b\n\x07\x41\x42ORTED\x10\x03\x12\x0b\n\x07STOPPED\x10\x04\x12\t\n\x05\x42\x45GIN\x10\x05\x12\x07\n\x03\x45ND\x10\x06\x62\x06proto3'
)

_LUASTATE = _descriptor.EnumDescriptor(
  name='LuaState',
  full_name='robotc.LuaState',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='IDLE', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='RUNNING', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='PAUSED', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='ABORTED', index=3, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='STOPPED', index=4, number=4,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='BEGIN', index=5, number=5,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='END', index=6, number=6,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=1219,
  serialized_end=1310,
)
_sym_db.RegisterEnumDescriptor(_LUASTATE)

LuaState = enum_type_wrapper.EnumTypeWrapper(_LUASTATE)
IDLE = 0
RUNNING = 1
PAUSED = 2
ABORTED = 3
STOPPED = 4
BEGIN = 5
END = 6



_RESPONSE = _descriptor.Descriptor(
  name='Response',
  full_name='robotc.Response',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='ret', full_name='robotc.Response.ret', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=26,
  serialized_end=49,
)


_COORDINATE = _descriptor.Descriptor(
  name='Coordinate',
  full_name='robotc.Coordinate',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='x', full_name='robotc.Coordinate.x', index=0,
      number=1, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='y', full_name='robotc.Coordinate.y', index=1,
      number=2, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='z', full_name='robotc.Coordinate.z', index=2,
      number=3, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=51,
  serialized_end=96,
)


_ROTATION = _descriptor.Descriptor(
  name='Rotation',
  full_name='robotc.Rotation',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='r', full_name='robotc.Rotation.r', index=0,
      number=1, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='p', full_name='robotc.Rotation.p', index=1,
      number=2, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='y', full_name='robotc.Rotation.y', index=2,
      number=3, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=98,
  serialized_end=141,
)


_PR = _descriptor.Descriptor(
  name='PR',
  full_name='robotc.PR',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='position', full_name='robotc.PR.position', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='rotation', full_name='robotc.PR.rotation', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=143,
  serialized_end=221,
)


_JPOSE = _descriptor.Descriptor(
  name='JPose',
  full_name='robotc.JPose',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='joints', full_name='robotc.JPose.joints', index=0,
      number=1, type=1, cpp_type=5, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=223,
  serialized_end=246,
)


_PAYLOAD = _descriptor.Descriptor(
  name='Payload',
  full_name='robotc.Payload',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='mass', full_name='robotc.Payload.mass', index=0,
      number=1, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='cog', full_name='robotc.Payload.cog', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=248,
  serialized_end=304,
)


_PAYLOADMASS = _descriptor.Descriptor(
  name='PayloadMass',
  full_name='robotc.PayloadMass',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='mass', full_name='robotc.PayloadMass.mass', index=0,
      number=1, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=306,
  serialized_end=333,
)


_PAYLOADCOG = _descriptor.Descriptor(
  name='PayloadCog',
  full_name='robotc.PayloadCog',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='cog', full_name='robotc.PayloadCog.cog', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=335,
  serialized_end=380,
)


_IOPIN = _descriptor.Descriptor(
  name='IOPin',
  full_name='robotc.IOPin',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='pin', full_name='robotc.IOPin.pin', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=382,
  serialized_end=402,
)


_DIO = _descriptor.Descriptor(
  name='DIO',
  full_name='robotc.DIO',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='pin', full_name='robotc.DIO.pin', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value', full_name='robotc.DIO.value', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=404,
  serialized_end=437,
)


_AIO = _descriptor.Descriptor(
  name='AIO',
  full_name='robotc.AIO',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='pin', full_name='robotc.AIO.pin', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='mode', full_name='robotc.AIO.mode', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='value', full_name='robotc.AIO.value', index=2,
      number=3, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=439,
  serialized_end=486,
)


_MODBUSEXTERNALIOSTATE = _descriptor.Descriptor(
  name='ModbusExternalIOState',
  full_name='robotc.ModbusExternalIOState',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='robotc.ModbusExternalIOState.id', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='state', full_name='robotc.ModbusExternalIOState.state', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=488,
  serialized_end=538,
)


_IO = _descriptor.Descriptor(
  name='IO',
  full_name='robotc.IO',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='robotDIOIn', full_name='robotc.IO.robotDIOIn', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='robotDIOOut', full_name='robotc.IO.robotDIOOut', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='robotAIOIn', full_name='robotc.IO.robotAIOIn', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='robotAIOOut', full_name='robotc.IO.robotAIOOut', index=3,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='tcpDIOIn', full_name='robotc.IO.tcpDIOIn', index=4,
      number=5, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='tcpDIOOut', full_name='robotc.IO.tcpDIOOut', index=5,
      number=6, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='extIODevices', full_name='robotc.IO.extIODevices', index=6,
      number=7, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=541,
  serialized_end=795,
)


_HARDWARE = _descriptor.Descriptor(
  name='Hardware',
  full_name='robotc.Hardware',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='model', full_name='robotc.Hardware.model', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='sn', full_name='robotc.Hardware.sn', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='driver_version', full_name='robotc.Hardware.driver_version', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=797,
  serialized_end=858,
)


_ROBOTINFO = _descriptor.Descriptor(
  name='RobotInfo',
  full_name='robotc.RobotInfo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='robot', full_name='robotc.RobotInfo.robot', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='flange', full_name='robotc.RobotInfo.flange', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='led', full_name='robotc.RobotInfo.led', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='joints', full_name='robotc.RobotInfo.joints', index=3,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='rcVersion', full_name='robotc.RobotInfo.rcVersion', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='name', full_name='robotc.RobotInfo.name', index=5,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='comboard', full_name='robotc.RobotInfo.comboard', index=6,
      number=7, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='mac', full_name='robotc.RobotInfo.mac', index=7,
      number=8, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='arm', full_name='robotc.RobotInfo.arm', index=8,
      number=9, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='appTag', full_name='robotc.RobotInfo.appTag', index=9,
      number=10, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='typeTag', full_name='robotc.RobotInfo.typeTag', index=10,
      number=11, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=861,
  serialized_end=1150,
)


_DEVICEINFO = _descriptor.Descriptor(
  name='DeviceInfo',
  full_name='robotc.DeviceInfo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='devInfo', full_name='robotc.DeviceInfo.devInfo', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='dev_type', full_name='robotc.DeviceInfo.dev_type', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1152,
  serialized_end=1217,
)

_PR.fields_by_name['position'].message_type = _COORDINATE
_PR.fields_by_name['rotation'].message_type = _ROTATION
_PAYLOAD.fields_by_name['cog'].message_type = _COORDINATE
_PAYLOADCOG.fields_by_name['cog'].message_type = _COORDINATE
_IO.fields_by_name['robotDIOIn'].message_type = _DIO
_IO.fields_by_name['robotDIOOut'].message_type = _DIO
_IO.fields_by_name['robotAIOIn'].message_type = _AIO
_IO.fields_by_name['robotAIOOut'].message_type = _AIO
_IO.fields_by_name['tcpDIOIn'].message_type = _DIO
_IO.fields_by_name['tcpDIOOut'].message_type = _DIO
_IO.fields_by_name['extIODevices'].message_type = _MODBUSEXTERNALIOSTATE
_ROBOTINFO.fields_by_name['robot'].message_type = _HARDWARE
_ROBOTINFO.fields_by_name['flange'].message_type = _HARDWARE
_ROBOTINFO.fields_by_name['led'].message_type = _HARDWARE
_ROBOTINFO.fields_by_name['joints'].message_type = _HARDWARE
_ROBOTINFO.fields_by_name['comboard'].message_type = _HARDWARE
_ROBOTINFO.fields_by_name['arm'].message_type = _HARDWARE
_DEVICEINFO.fields_by_name['devInfo'].message_type = _HARDWARE
DESCRIPTOR.message_types_by_name['Response'] = _RESPONSE
DESCRIPTOR.message_types_by_name['Coordinate'] = _COORDINATE
DESCRIPTOR.message_types_by_name['Rotation'] = _ROTATION
DESCRIPTOR.message_types_by_name['PR'] = _PR
DESCRIPTOR.message_types_by_name['JPose'] = _JPOSE
DESCRIPTOR.message_types_by_name['Payload'] = _PAYLOAD
DESCRIPTOR.message_types_by_name['PayloadMass'] = _PAYLOADMASS
DESCRIPTOR.message_types_by_name['PayloadCog'] = _PAYLOADCOG
DESCRIPTOR.message_types_by_name['IOPin'] = _IOPIN
DESCRIPTOR.message_types_by_name['DIO'] = _DIO
DESCRIPTOR.message_types_by_name['AIO'] = _AIO
DESCRIPTOR.message_types_by_name['ModbusExternalIOState'] = _MODBUSEXTERNALIOSTATE
DESCRIPTOR.message_types_by_name['IO'] = _IO
DESCRIPTOR.message_types_by_name['Hardware'] = _HARDWARE
DESCRIPTOR.message_types_by_name['RobotInfo'] = _ROBOTINFO
DESCRIPTOR.message_types_by_name['DeviceInfo'] = _DEVICEINFO
DESCRIPTOR.enum_types_by_name['LuaState'] = _LUASTATE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Response = _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
  'DESCRIPTOR' : _RESPONSE,
  '__module__' : 'messages_pb2'
  # @@protoc_insertion_point(class_scope:robotc.Response)
  })
_sym_db.RegisterMessage(Response)

Coordinate = _reflection.GeneratedProtocolMessageType('Coordinate', (_message.Message,), {
  'DESCRIPTOR' : _COORDINATE,
  '__module__' : 'messages_pb2'
  # @@protoc_insertion_point(class_scope:robotc.Coordinate)
  })
_sym_db.RegisterMessage(Coordinate)

Rotation = _reflection.GeneratedProtocolMessageType('Rotation', (_message.Message,), {
  'DESCRIPTOR' : _ROTATION,
  '__module__' : 'messages_pb2'
  # @@protoc_insertion_point(class_scope:robotc.Rotation)
  })
_sym_db.RegisterMessage(Rotation)

PR = _reflection.GeneratedProtocolMessageType('PR', (_message.Message,), {
  'DESCRIPTOR' : _PR,
  '__module__' : 'messages_pb2'
  # @@protoc_insertion_point(class_scope:robotc.PR)
  })
_sym_db.RegisterMessage(PR)

JPose = _reflection.GeneratedProtocolMessageType('JPose', (_message.Message,), {
  'DESCRIPTOR' : _JPOSE,
  '__module__' : 'messages_pb2'
  # @@protoc_insertion_point(class_scope:robotc.JPose)
  })
_sym_db.RegisterMessage(JPose)

Payload = _reflection.GeneratedProtocolMessageType('Payload', (_message.Message,), {
  'DESCRIPTOR' : _PAYLOAD,
  '__module__' : 'messages_pb2'
  # @@protoc_insertion_point(class_scope:robotc.Payload)
  })
_sym_db.RegisterMessage(Payload)

PayloadMass = _reflection.GeneratedProtocolMessageType('PayloadMass', (_message.Message,), {
  'DESCRIPTOR' : _PAYLOADMASS,
  '__module__' : 'messages_pb2'
  # @@protoc_insertion_point(class_scope:robotc.PayloadMass)
  })
_sym_db.RegisterMessage(PayloadMass)

PayloadCog = _reflection.GeneratedProtocolMessageType('PayloadCog', (_message.Message,), {
  'DESCRIPTOR' : _PAYLOADCOG,
  '__module__' : 'messages_pb2'
  # @@protoc_insertion_point(class_scope:robotc.PayloadCog)
  })
_sym_db.RegisterMessage(PayloadCog)

IOPin = _reflection.GeneratedProtocolMessageType('IOPin', (_message.Message,), {
  'DESCRIPTOR' : _IOPIN,
  '__module__' : 'messages_pb2'
  # @@protoc_insertion_point(class_scope:robotc.IOPin)
  })
_sym_db.RegisterMessage(IOPin)

DIO = _reflection.GeneratedProtocolMessageType('DIO', (_message.Message,), {
  'DESCRIPTOR' : _DIO,
  '__module__' : 'messages_pb2'
  # @@protoc_insertion_point(class_scope:robotc.DIO)
  })
_sym_db.RegisterMessage(DIO)

AIO = _reflection.GeneratedProtocolMessageType('AIO', (_message.Message,), {
  'DESCRIPTOR' : _AIO,
  '__module__' : 'messages_pb2'
  # @@protoc_insertion_point(class_scope:robotc.AIO)
  })
_sym_db.RegisterMessage(AIO)

ModbusExternalIOState = _reflection.GeneratedProtocolMessageType('ModbusExternalIOState', (_message.Message,), {
  'DESCRIPTOR' : _MODBUSEXTERNALIOSTATE,
  '__module__' : 'messages_pb2'
  # @@protoc_insertion_point(class_scope:robotc.ModbusExternalIOState)
  })
_sym_db.RegisterMessage(ModbusExternalIOState)

IO = _reflection.GeneratedProtocolMessageType('IO', (_message.Message,), {
  'DESCRIPTOR' : _IO,
  '__module__' : 'messages_pb2'
  # @@protoc_insertion_point(class_scope:robotc.IO)
  })
_sym_db.RegisterMessage(IO)

Hardware = _reflection.GeneratedProtocolMessageType('Hardware', (_message.Message,), {
  'DESCRIPTOR' : _HARDWARE,
  '__module__' : 'messages_pb2'
  # @@protoc_insertion_point(class_scope:robotc.Hardware)
  })
_sym_db.RegisterMessage(Hardware)

RobotInfo = _reflection.GeneratedProtocolMessageType('RobotInfo', (_message.Message,), {
  'DESCRIPTOR' : _ROBOTINFO,
  '__module__' : 'messages_pb2'
  # @@protoc_insertion_point(class_scope:robotc.RobotInfo)
  })
_sym_db.RegisterMessage(RobotInfo)

DeviceInfo = _reflection.GeneratedProtocolMessageType('DeviceInfo', (_message.Message,), {
  'DESCRIPTOR' : _DEVICEINFO,
  '__module__' : 'messages_pb2'
  # @@protoc_insertion_point(class_scope:robotc.DeviceInfo)
  })
_sym_db.RegisterMessage(DeviceInfo)


# @@protoc_insertion_point(module_scope)



import grpc
import asyncio
from enum import Enum, unique

from .pb2 import messages_pb2 as msg
from google.protobuf.empty_pb2 import Empty
from .pb2 import robot_controller_pb2_grpc
from .pb2 import robot_controller_pb2 as rc
from .pb2 import private_controller_pb2_grpc
from .pb2 import private_controller_pb2 as pc

class RobotState(Enum):
    '''机器人状态
    '''
    DISCONNECTED = 0  # 已断开连接
    ESTOP = 1         # 急停停止状态
    BOOTING = 2       # 启动中
    ROBOT_OFF = 3     # 电源关闭
    ROBOT_ON = 4      # 电源开启
    IDLE = 5          # 空闲中
    PAUSED = 6        # 暂停中
    RUNNING = 7       # 机器人运动运行中
    UPDATING = 8      # 更新固件中
    STARTING = 9      # 启动中
    STOPPING = 10     # 停止中
    TEACHING = 11     # 示教中
    STOP = 12         # 普通停止
    FINETUNING = 13   # 微调中

class CartesianPose:
    '''空间位置
    
    笛卡尔坐标描述的位姿
    '''
    def __init__(self, x=0, y=0, z=0, rz=0, ry=0, rx=0, base=None):
        if type(x) is msg.PR:
            self.pos = (x.position.x, x.position.y, x.position.z, x.rotation.r, x.rotation.p, x.rotation.y)
        elif type(x) is CartesianPose:
            self.pos = x.pos[:]
            base = x.base
        else:
            self.pos = (x, y, z, rz, ry, rx)
        self.is_joint = False
        # 要求 base 是元组或列表而不是 CartesianPose 对象
        self.base = getattr(base, 'pos', base) if base is not None else None

    def __str__(self):
        if self.base is None:
            return 'CartesianPose' + str(self.pos)
        else:
            return f'CartesianPose({self.pos[0]}, {self.pos[1]}, {self.pos[2]}, {self.pos[3]}, {self.pos[4]}, {self.pos[5]}, base={self.base})'

    def __getattr__(self, key):
        pos_idx_arr = ['x', 'y', 'z', 'rz', 'ry', 'rx']
        if key in pos_idx_arr:
            idx = pos_idx_arr.index(key)
            return getattr(self, 'pos')[idx]
        else:
            return getattr(self, key)

    def __eq__(self, other):
        return self.pos == other.pos and self.base == other.base

    def to_PR(self):
        p = msg.Coordinate(x=self.pos[0], y=self.pos[1], z=self.pos[2])
        r = msg.Rotation(r=self.pos[3], p=self.pos[4], y=self.pos[5])
        return msg.PR(position=p,rotation=r)

    def base_set_PR(self, pr):
        if self.base is not None:
            pr.position.x = self.base[0]
            pr.position.y = self.base[1]
            pr.position.z = self.base[2]
            pr.rotation.r = self.base[3]
            pr.rotation.p = self.base[4]
            pr.rotation.y = self.base[5]

class JointPose:
    '''关节位置
    
    关节旋转角度描述的机器人姿态
    '''
    def __init__(self, *j):
        self.pos = j
        self.is_joint = True

    def __str__(self):
        return 'JointPose' + str(self.pos)

class LebaiRobot:
    '''
    :param ip: 机器人设备 IP

    :returns: 返回一个乐白机器人控制实例
    '''
    def __init__(self, ip):
        self.rcc = grpc.aio.insecure_channel(f'{ip}:5181')
        self.rcs = robot_controller_pb2_grpc.RobotControllerStub(self.rcc)

        self.pcc = grpc.aio.insecure_channel(f'{ip}:5182')
        self.pcs = private_controller_pb2_grpc.RobotPrivateControllerStub(self.pcc)

    async def start_sys(self):
        await self.rcs.StartSys(Empty())

    async def stop_sys(self):
        await self.rcs.StopSys(Empty())

    async def powerdown(self):
        await self.rcs.PowerDown(Empty)

    async def stop(self):
        await self.rcs.Stop(Empty())

    async def estop(self):
        await self.rcs.EStop(Empty())

    async def teach_mode(self):
        await self.rcs.TeachMode(Empty)

    async def end_teach_mode(self):
        await self.rcs.EndTeachMode(Empty)

    async def resume(self):
        await self.rcs.Resume(Empty())

    async def pause(self):
        await self.rcs.Pause(Empty())

    async def get_robot_mode(self):
        res = await self.rcs.GetRobotMode(Empty())
        return RobotState(res.mode)

    async def get_velocity_factor(self):
        res = await self.rcs.GetVelocityFactor(Empty())
        return res.value

    async def set_velocity_factor(self, factor):
        await self.rcs.SetVelocityFactor(rc.Factor(value=factor))

    async def set_gravity(self, x=0, y=0, z=-9.8):
        if type(x) is tuple or type(x) is list:
            y = x[1]
            z = x[2]
            x = x[0]
        await self.rcs.SetGravity(msg.Coordinate(x=x,y=y,z=z))

    async def get_gravity(self):
        res = await self.rcs.GetGravity(Empty())
        return (res.x, res.y, res.z)

    async def set_payload(self, x=0, y=0, z=-9.8, mass=0):
        if type(x) is tuple:
            if type(y) is not tuple and type(x[0]) is tuple:
                y = x[1]
                x = x[0]
            mass = y
            z = x[2]
            y = x[1]
            x = x[0]
        await self.rcs.SetPayload(msg.Payload(mass=mass, cog=msg.Coordinate(x=x,y=y,z=z)))

    async def get_payload(self):
        res = await self.rcs.GetPayload(Empty())
        return ((res.cog.x, res.cog.y, res.cog.z), res.mass)

    async def set_payload_mass(self, mass):
        await self.rcs.SetPayloadMass(msg.PayloadMass(mass=mass))

    async def get_payload_mass(self):
        res = await self.rcs.GetPayloadMass(Empty())
        return res.mass

    async def set_payload_cog(self, x=0, y=0, z=0):
        if type(x) is tuple or type(x) is list:
            y = x[1]
            z = x[2]
            x = x[0]
        await self.rcs.SetPayloadCog(msg.PayloadCog(cog=msg.Coordinate(x=x,y=y,z=z)))

    async def get_payload_cog(self):
        res = await self.rcs.GetPayloadCog(Empty())
        return (res.cog.x, res.cog.y, res.cog.z)

    async def set_tcp(self, x=0, y=0, z=0, rz=0, ry=0, rx=0):
        tcp = CartesianPose(x, y, z, rz, ry, rx)
        await self.rcs.SetTcp(tcp.to_PR())

    async def get_tcp(self):
        res = await self.rcs.GetTcp(Empty())
        return CartesianPose(res)

    async def get_claw_aio(self, pin):
        pin = pin.lower()
        if pin == 'force':
            res = await self.rcs.GetClawHoldOn(Empty())
            return res.hold_on
        elif pin == 'weight':
            res = await self.rcs.GetClawWeight(Empty())
            return res.weight
        else: # pin == 'amplitude':
            res = await self.rcs.GetClawAmplitude(Empty())
            return res.amplitude

    async def set_claw_aio(self, pin, value=0):
        pin = pin.lower()
        if pin == 'force':
            await self.rcs.SetClawForce(rc.Force(force=value))
        else: # pin == 'amplitude':
            await self.rcs.SetClawAmplitude(rc.Amplitude(amplitude=value))

    async def set_claw(self, force=0, amplitude=0):
        # TODO: RC
        pass

    async def movej(self, p, a=0, v=0, t=0, r=0):
        '''线性移动（关节空间）

        :param p:
            - `JointPose` 关节位置
            - `CartesianPose` 空间位置（将通过运动学反解转为关节位置）
        :param a: 主轴的关节加速度 (rad/s\ :sup:`2`)
        :param v: 主轴的关节速度 (rad/s)
        :param t: 运动时间 (s)
        :param r: 交融半径 (m)
        '''
        req = rc.MoveJRequest(
            joint_pose_to = list(p.pos),
            pose_is_joint_angle=getattr(p, 'is_joint', True),
            acceleration=a,
            velocity=v,
            time=t,
            blend_radius=r
        )
        if type(p) is CartesianPose:
            p.base_set_PR(req.pose_base)
        await self.rcs.MoveJ(req)

    async def movel(self, p, a=0, v=0, t=0, r=0):
        req = rc.MoveLRequest(
            pose_to = list(p.pos),
            pose_is_joint_angle=getattr(p, 'is_joint', False),
            acceleration=a,
            velocity=v,
            time=t,
            blend_radius=r
        )
        if type(p) is CartesianPose:
            p.base_set_PR(req.pose_base)
        await self.rcs.MoveL(req)

    async def movec(self, via, p, rad=0, a=0, v=0, t=0, r=0):
        req = rc.MoveCRequest(
            pose_via = list(via.pos),
            pose_via_is_joint=getattr(via, 'is_joint', False),
            pose_to = list(p.pos),
            pose_to_is_joint=getattr(p, 'is_joint', False),
            acceleration=a,
            velocity=v,
            time=t,
            blend_radius=r,
            rad=rad
        )
        if type(p) is CartesianPose:
            p.base_set_PR(req.pose_base)
        await self.rcs.MoveC(req)

    async def stop_move(self):
        await self.rcs.StopMove(Empty())

    async def movej_until(self):
        pass

    async def movej_until_rt(self):
        pass

    async def movel_until(self):
        pass

    async def movel_until_rt(self):
        pass

    async def movec_until(self):
        pass

    async def movec_until_rt(self):
        pass

    async def kinematics_forward(self):
        pass

    async def kinematics_inverse(self):
        pass

    async def pose_times(self):
        pass

    async def pose_inverse(self):
        pass

    async def get_actual_joint_positions(self):
        '''获取实际关节位置

        :returns: `JointPose` 关节位置
        '''
        res = await self.rcs.GetActualJointPositions(Empty())
        return JointPose(*res.joints)

    async def get_target_joint_positions(self):
        res = await self.rcs.GetTargetJointPositions(Empty())
        return JointPose(*res.joints)

    async def get_actual_joint_speeds(self):
        res = await self.rcs.GetActualJointSpeeds(Empty())
        return tuple(res.joints)

    async def get_target_joint_speeds(self):
        res = await self.rcs.GetTargetJointSpeeds(Empty())
        return tuple(res.joints)

    async def get_joint_torques(self):
        res = await self.rcs.GetJointTorques(Empty())
        return tuple(res.joints)

    async def get_actual_joint_torques(self):
        # TODO: RC
        pass

    async def get_target_joint_torques(self):
        # TODO: RC
        pass

    async def get_joint_temperatures(self):
        # TODO: RC
        pass

    async def get_joint_temp(self, joint):
        '''获取关节温度

        :param joint: 关节序号

        :returns: 关节当前温度 (℃)
        '''
        res = await self.rcs.GetJointTemp(rc.IntRequest(index=joint))
        return res.degree

    async def get_actual_tcp_pose(self):
        '''获取实际空间位置

        :returns: `CartesianPose` 空间位置
        '''
        res = await self.rcs.GetActualTcpPose(Empty())
        return CartesianPose(*res.vector)

    async def get_target_tcp_pose(self):
        res = await self.rcs.GetTargetTcpPose(Empty())
        return CartesianPose(*res.vector)

    async def get_robot_poses(self):
        pass

    async def get_robot_data(self):
        pass

    async def write_extra_servo_param(self):
        pass

    async def read_extra_servo_param(self):
        pass

    async def reset_extra_servo_param(self):
        pass

    async def write_extra_servo_params(self):
        pass

    async def read_extra_servo_params(self):
        pass

    async def reset_extra_servo_params(self):
        pass

    async def write_joint_backlash(self):
        pass

    async def read_joint_backlash(self):
        pass

    async def reset_joint_backlash(self):
        pass

    async def write_joint_backlashes(self):
        pass

    async def read_joint_backlashes(self):
        pass

    async def reset_joint_backlashes(self):
        pass

    async def read_enable_joint_backlash(self):
        pass

    async def read_enable_joint_backlashes(self):
        pass

    async def write_enable_joint_backlash(self):
        pass

    async def write_enable_joint_backlashes(self):
        pass

    async def write_joint_backlash_param(self):
        pass

    async def read_joint_backlash_param(self):
        pass

    async def reset_joint_backlash_param(self):
        pass

    async def write_joint_backlash_params(self):
        pass

    async def read_joint_backlash_params(self):
        pass

    async def reset_joint_backlash_params(self):
        pass

    async def set_dio(self, pin, value):
        await self.rcs.SetDIO(msg.DIO(pin=pin, value=value))

    async def get_dio(self, pin):
        res = await self.rcs.GetDIO(msg.IOPin(pin=pin))
        return res.value

    async def set_aio(self):
        pass

    async def get_aio(self):
        pass

    async def set_tcp_dio(self):
        pass

    async def get_tcp_dio(self):
        pass

    async def set_tcp_aio(self):
        pass

    async def get_tcp_aio(self):
        pass

    async def set_external_do(self):
        pass

    async def set_external_dos(self):
        pass

    async def get_external_do(self):
        pass

    async def get_external_dos(self):
        pass

    async def get_external_di(self):
        pass

    async def get_external_dis(self):
        pass

    async def set_external_ao(self):
        pass

    async def set_external_aos(self):
        pass

    async def get_external_ao(self):
        pass

    async def get_external_aos(self):
        pass

    async def get_external_ai(self):
        pass

    async def get_external_ais(self):
        pass

    async def get_external_ios(self):
        pass

    async def set_led(self):
        pass

    async def set_voice(self):
        pass

    async def set_fan(self):
        pass

    async def set_signal(self):
        pass

    async def get_signal(self):
        pass

    async def add_signal(self):
        pass

    async def enable_joint_limits(self):
        await self.pcs.EnableJointLimit(pc.TrueOrFalse(val=True))

    async def disable_joint_limits(self):
        await self.pcs.EnableJointLimit(pc.TrueOrFalse(val=False))
